@@ -7,22 +7,16 @@ use std::{
 
 use crate::{
     backtest::{
+        BacktestError,
         assettype::AssetType,
         models::{FeeModel, LatencyModel, QueueModel},
-        order::OrderBus,
+        order::ExchToLocal,
         proc::Processor,
         state::State,
-        BacktestError,
     },
-    depth::{L2MarketDepth, MarketDepth, INVALID_MAX, INVALID_MIN},
+    depth::{INVALID_MAX, INVALID_MIN, L2MarketDepth, MarketDepth},
     prelude::OrdType,
     types::{
-        Event,
-        Order,
-        OrderId,
-        Side,
-        Status,
-        TimeInForce,
         EXCH_ASK_DEPTH_CLEAR_EVENT,
         EXCH_ASK_DEPTH_EVENT,
         EXCH_ASK_DEPTH_SNAPSHOT_EVENT,
@@ -33,6 +27,12 @@ use crate::{
         EXCH_DEPTH_CLEAR_EVENT,
         EXCH_EVENT,
         EXCH_SELL_TRADE_EVENT,
+        Event,
+        Order,
+        OrderId,
+        Side,
+        Status,
+        TimeInForce,
     },
 };
 
@@ -75,12 +75,10 @@ where
     buy_orders: HashMap<i64, HashSet<OrderId>>,
     sell_orders: HashMap<i64, HashSet<OrderId>>,
 
-    orders_to: OrderBus,
-    orders_from: OrderBus,
+    order_e2l: ExchToLocal<LM>,
 
     depth: MD,
     state: State<AT, FM>,
-    order_latency: LM,
     queue_model: QM,
 
     filled_orders: Vec<OrderId>,
@@ -98,45 +96,21 @@ where
     pub fn new(
         depth: MD,
         state: State<AT, FM>,
-        order_latency: LM,
         queue_model: QM,
-        orders_to: OrderBus,
-        orders_from: OrderBus,
+        order_e2l: ExchToLocal<LM>,
     ) -> Self {
         Self {
             orders: Default::default(),
             buy_orders: Default::default(),
             sell_orders: Default::default(),
-            orders_to,
-            orders_from,
+            order_e2l,
             depth,
             state,
-            order_latency,
             queue_model,
             filled_orders: Default::default(),
         }
     }
 
-    fn process_recv_order_(
-        &mut self,
-        mut order: Order,
-        recv_timestamp: i64,
-    ) -> Result<(), BacktestError> {
-        // Processes a new order.
-        if order.req == Status::New {
-            order.req = Status::None;
-            self.ack_new(order, recv_timestamp)?;
-        }
-        // Processes a cancel order.
-        else if order.req == Status::Canceled {
-            order.req = Status::None;
-            self.ack_cancel(order, recv_timestamp)?;
-        } else {
-            return Err(BacktestError::InvalidOrderRequest);
-        }
-        Ok(())
-    }
-
     fn check_if_sell_filled(
         &mut self,
         order: &mut Order,
@@ -148,14 +122,14 @@ where
             Ordering::Greater => {}
             Ordering::Less => {
                 self.filled_orders.push(order.order_id);
-                return self.fill(order, timestamp, true, order.price_tick);
+                return self.fill::<true>(order, timestamp, true, order.price_tick);
             }
             Ordering::Equal => {
                 // Updates the order's queue position.
                 self.queue_model.trade(order, qty, &self.depth);
                 if self.queue_model.is_filled(order, &self.depth) > 0.0 {
                     self.filled_orders.push(order.order_id);
-                    return self.fill(order, timestamp, true, order.price_tick);
+                    return self.fill::<true>(order, timestamp, true, order.price_tick);
                 }
             }
         }
@@ -172,7 +146,7 @@ where
         match order.price_tick.cmp(&price_tick) {
             Ordering::Greater => {
                 self.filled_orders.push(order.order_id);
-                return self.fill(order, timestamp, true, order.price_tick);
+                return self.fill::<true>(order, timestamp, true, order.price_tick);
             }
             Ordering::Less => {}
             Ordering::Equal => {
@@ -180,14 +154,14 @@ where
                 self.queue_model.trade(order, qty, &self.depth);
                 if self.queue_model.is_filled(order, &self.depth) > 0.0 {
                     self.filled_orders.push(order.order_id);
-                    return self.fill(order, timestamp, true, order.price_tick);
+                    return self.fill::<true>(order, timestamp, true, order.price_tick);
                 }
             }
         }
         Ok(())
     }
 
-    fn fill(
+    fn fill<const MAKE_RESPONSE: bool>(
         &mut self,
         order: &mut Order,
         timestamp: i64,
@@ -212,11 +186,12 @@ where
         order.leaves_qty = 0.0;
         order.status = Status::Filled;
         order.exch_timestamp = timestamp;
-        let local_recv_timestamp =
-            order.exch_timestamp + self.order_latency.response(timestamp, order);
 
         self.state.apply_fill(order);
-        self.orders_to.append(order.clone(), local_recv_timestamp);
+
+        if MAKE_RESPONSE {
+            self.order_e2l.respond(order.clone());
+        }
         Ok(())
     }
 
@@ -281,7 +256,7 @@ where
                 for (_, order) in orders_borrowed.iter_mut() {
                     if order.side == Side::Sell && order.price_tick <= new_best_tick {
                         self.filled_orders.push(order.order_id);
-                        self.fill(order, timestamp, true, order.price_tick)?;
+                        self.fill::<true>(order, timestamp, true, order.price_tick)?;
                     }
                 }
             } else {
@@ -290,7 +265,7 @@ where
                         for order_id in order_ids.clone().iter() {
                             self.filled_orders.push(*order_id);
                             let order = orders_borrowed.get_mut(order_id).unwrap();
-                            self.fill(order, timestamp, true, order.price_tick)?;
+                            self.fill::<true>(order, timestamp, true, order.price_tick)?;
                         }
                     }
                 }
@@ -317,7 +292,7 @@ where
                 for (_, order) in orders_borrowed.iter_mut() {
                     if order.side == Side::Buy && order.price_tick >= new_best_tick {
                         self.filled_orders.push(order.order_id);
-                        self.fill(order, timestamp, true, order.price_tick)?;
+                        self.fill::<true>(order, timestamp, true, order.price_tick)?;
                     }
                 }
             } else {
@@ -326,7 +301,7 @@ where
                         for order_id in order_ids.clone().iter() {
                             self.filled_orders.push(*order_id);
                             let order = orders_borrowed.get_mut(order_id).unwrap();
-                            self.fill(order, timestamp, true, order.price_tick)?;
+                            self.fill::<true>(order, timestamp, true, order.price_tick)?;
                         }
                     }
                 }
@@ -336,7 +311,7 @@ where
         Ok(())
     }
 
-    fn ack_new(&mut self, mut order: Order, timestamp: i64) -> Result<(), BacktestError> {
+    fn ack_new(&mut self, order: &mut Order, timestamp: i64) -> Result<(), BacktestError> {
         if self.orders.borrow().contains_key(&order.order_id) {
             return Err(BacktestError::OrderIdExist);
         }
@@ -349,18 +324,19 @@ where
                         match order.time_in_force {
                             TimeInForce::GTX => {
                                 order.status = Status::Expired;
-
                                 order.exch_timestamp = timestamp;
-                                let local_recv_timestamp =
-                                    timestamp + self.order_latency.response(timestamp, &order);
-                                self.orders_to.append(order.clone(), local_recv_timestamp);
                                 Ok(())
                             }
                             TimeInForce::GTC | TimeInForce::FOK | TimeInForce::IOC => {
                                 // Since this always fills the full quantity, both FOK and IOC
                                 // orders are also fully filled at the best price.
                                 // Takes the market.
-                                self.fill(&mut order, timestamp, false, self.depth.best_ask_tick())
+                                self.fill::<false>(
+                                    order,
+                                    timestamp,
+                                    false,
+                                    self.depth.best_ask_tick(),
+                                )
                             }
                             TimeInForce::Unsupported => Err(BacktestError::InvalidOrderRequest),
                         }
@@ -368,7 +344,7 @@ where
                         match order.time_in_force {
                             TimeInForce::GTC | TimeInForce::GTX => {
                                 // Initializes the order's queue position.
-                                self.queue_model.new_order(&mut order, &self.depth);
+                                self.queue_model.new_order(order, &self.depth);
                                 order.status = Status::New;
                                 // The exchange accepts this order.
                                 self.buy_orders
@@ -377,19 +353,14 @@ where
                                     .insert(order.order_id);
 
                                 order.exch_timestamp = timestamp;
-                                let local_recv_timestamp =
-                                    timestamp + self.order_latency.response(timestamp, &order);
-                                self.orders_to.append(order.clone(), local_recv_timestamp);
-                                self.orders.borrow_mut().insert(order.order_id, order);
+                                self.orders
+                                    .borrow_mut()
+                                    .insert(order.order_id, order.clone());
                                 Ok(())
                             }
                             TimeInForce::FOK | TimeInForce::IOC => {
                                 order.status = Status::Expired;
-
                                 order.exch_timestamp = timestamp;
-                                let local_recv_timestamp =
-                                    timestamp + self.order_latency.response(timestamp, &order);
-                                self.orders_to.append(order.clone(), local_recv_timestamp);
                                 Ok(())
                             }
                             TimeInForce::Unsupported => Err(BacktestError::InvalidOrderRequest),
@@ -398,7 +369,7 @@ where
                 }
                 OrdType::Market => {
                     // Takes the market.
-                    self.fill(&mut order, timestamp, false, self.depth.best_ask_tick())
+                    self.fill::<false>(order, timestamp, false, self.depth.best_ask_tick())
                 }
                 OrdType::Unsupported => Err(BacktestError::InvalidOrderRequest),
             }
@@ -410,18 +381,19 @@ where
                         match order.time_in_force {
                             TimeInForce::GTX => {
                                 order.status = Status::Expired;
-
                                 order.exch_timestamp = timestamp;
-                                let local_recv_timestamp =
-                                    timestamp + self.order_latency.response(timestamp, &order);
-                                self.orders_to.append(order.clone(), local_recv_timestamp);
                                 Ok(())
                             }
                             TimeInForce::GTC | TimeInForce::FOK | TimeInForce::IOC => {
                                 // Since this always fills the full quantity, both FOK and IOC
                                 // orders are also fully filled at the best price.
                                 // Takes the market.
-                                self.fill(&mut order, timestamp, false, self.depth.best_bid_tick())
+                                self.fill::<false>(
+                                    order,
+                                    timestamp,
+                                    false,
+                                    self.depth.best_bid_tick(),
+                                )
                             }
                             TimeInForce::Unsupported => Err(BacktestError::InvalidOrderRequest),
                         }
@@ -429,7 +401,7 @@ where
                         match order.time_in_force {
                             TimeInForce::GTC | TimeInForce::GTX => {
                                 // Initializes the order's queue position.
-                                self.queue_model.new_order(&mut order, &self.depth);
+                                self.queue_model.new_order(order, &self.depth);
                                 order.status = Status::New;
                                 // The exchange accepts this order.
                                 self.sell_orders
@@ -438,19 +410,14 @@ where
                                     .insert(order.order_id);
 
                                 order.exch_timestamp = timestamp;
-                                let local_recv_timestamp =
-                                    timestamp + self.order_latency.response(timestamp, &order);
-                                self.orders_to.append(order.clone(), local_recv_timestamp);
-                                self.orders.borrow_mut().insert(order.order_id, order);
+                                self.orders
+                                    .borrow_mut()
+                                    .insert(order.order_id, order.clone());
                                 Ok(())
                             }
                             TimeInForce::FOK | TimeInForce::IOC => {
                                 order.status = Status::Expired;
-
                                 order.exch_timestamp = timestamp;
-                                let local_recv_timestamp =
-                                    timestamp + self.order_latency.response(timestamp, &order);
-                                self.orders_to.append(order.clone(), local_recv_timestamp);
                                 Ok(())
                             }
                             TimeInForce::Unsupported => Err(BacktestError::InvalidOrderRequest),
@@ -459,14 +426,14 @@ where
                 }
                 OrdType::Market => {
                     // Takes the market.
-                    self.fill(&mut order, timestamp, false, self.depth.best_bid_tick())
+                    self.fill::<false>(order, timestamp, false, self.depth.best_bid_tick())
                 }
                 OrdType::Unsupported => Err(BacktestError::InvalidOrderRequest),
             }
         }
     }
 
-    fn ack_cancel(&mut self, mut order: Order, timestamp: i64) -> Result<(), BacktestError> {
+    fn ack_cancel(&mut self, order: &mut Order, timestamp: i64) -> Result<(), BacktestError> {
         let exch_order = {
             let mut order_borrowed = self.orders.borrow_mut();
             order_borrowed.remove(&order.order_id)
@@ -475,172 +442,71 @@ where
         if exch_order.is_none() {
             order.req = Status::Rejected;
             order.exch_timestamp = timestamp;
-            let local_recv_timestamp = timestamp + self.order_latency.response(timestamp, &order);
-            self.orders_to.append(order, local_recv_timestamp);
             return Ok(());
         }
 
+        let exch_order = exch_order.unwrap();
+        let _ = std::mem::replace(order, exch_order);
+
         // Deletes the order.
-        let mut exch_order = exch_order.unwrap();
-        if exch_order.side == Side::Buy {
+        if order.side == Side::Buy {
             self.buy_orders
-                .get_mut(&exch_order.price_tick)
+                .get_mut(&order.price_tick)
                 .unwrap()
-                .remove(&exch_order.order_id);
+                .remove(&order.order_id);
         } else {
             self.sell_orders
-                .get_mut(&exch_order.price_tick)
+                .get_mut(&order.price_tick)
                 .unwrap()
-                .remove(&exch_order.order_id);
+                .remove(&order.order_id);
         }
-
-        // Makes the response.
-        exch_order.status = Status::Canceled;
-        exch_order.exch_timestamp = timestamp;
-        let local_recv_timestamp = timestamp + self.order_latency.response(timestamp, &exch_order);
-        self.orders_to
-            .append(exch_order.clone(), local_recv_timestamp);
+        order.status = Status::Canceled;
+        order.exch_timestamp = timestamp;
         Ok(())
     }
 
-    fn ack_modify(&mut self, mut order: Order, timestamp: i64) -> Result<(), BacktestError> {
-        let mut exch_order = {
-            let mut order_borrowed = self.orders.borrow_mut();
-            let exch_order = order_borrowed.remove(&order.order_id);
+    fn ack_modify<const RESET_QUEUE_POS: bool>(
+        &mut self,
+        order: &mut Order,
+        timestamp: i64,
+    ) -> Result<(), BacktestError> {
+        let (prev_order_price_tick, prev_leaves_qty) = {
+            let order_borrowed = self.orders.borrow();
+            let exch_order = order_borrowed.get(&order.order_id);
 
             // The order can be already deleted due to fill or expiration.
             if exch_order.is_none() {
                 order.req = Status::Rejected;
                 order.exch_timestamp = timestamp;
-                let local_recv_timestamp =
-                    timestamp + self.order_latency.response(timestamp, &order);
-                self.orders_to.append(order, local_recv_timestamp);
                 return Ok(());
             }
 
-            exch_order.unwrap()
+            let exch_order = exch_order.unwrap();
+            (exch_order.price_tick, exch_order.leaves_qty)
         };
 
-        let prev_price_tick = exch_order.price_tick;
-        exch_order.price_tick = order.price_tick;
-        // No partial fill occurs.
-        exch_order.qty = order.qty;
         // The initialization of the order queue position may not occur when the modified quantity
         // is smaller than the previous quantity, depending on the exchanges. It may need to
         // implement exchange-specific specialization.
-        let init_q_pos = true;
-
-        if exch_order.side == Side::Buy {
-            // Checks if the buy order price is greater than or equal to the current best ask.
-            if exch_order.price_tick >= self.depth.best_ask_tick() {
-                self.buy_orders
-                    .get_mut(&prev_price_tick)
-                    .unwrap()
-                    .remove(&exch_order.order_id);
-
-                if exch_order.time_in_force == TimeInForce::GTX {
-                    exch_order.status = Status::Expired;
-                } else {
-                    // Takes the market.
-                    return self.fill(
-                        &mut exch_order,
-                        timestamp,
-                        false,
-                        self.depth.best_ask_tick(),
-                    );
-                }
-
-                exch_order.exch_timestamp = timestamp;
-                let local_recv_timestamp =
-                    timestamp + self.order_latency.response(timestamp, &exch_order);
-                self.orders_to
-                    .append(exch_order.clone(), local_recv_timestamp);
-                Ok(())
-            } else {
-                // The exchange accepts this order.
-                if prev_price_tick != exch_order.price_tick {
-                    self.buy_orders
-                        .get_mut(&prev_price_tick)
-                        .unwrap()
-                        .remove(&exch_order.order_id);
-                    self.buy_orders
-                        .entry(exch_order.price_tick)
-                        .or_default()
-                        .insert(exch_order.order_id);
-                }
-                if init_q_pos || prev_price_tick != exch_order.price_tick {
-                    // Initializes the order's queue position.
-                    self.queue_model.new_order(&mut exch_order, &self.depth);
-                }
-                exch_order.status = Status::New;
-
-                exch_order.exch_timestamp = timestamp;
-                let local_recv_timestamp =
-                    timestamp + self.order_latency.response(timestamp, &exch_order);
-                self.orders_to
-                    .append(exch_order.clone(), local_recv_timestamp);
-
-                let mut order_borrowed = self.orders.borrow_mut();
-                order_borrowed.insert(exch_order.order_id, exch_order);
-
-                Ok(())
-            }
+        if RESET_QUEUE_POS
+            || prev_order_price_tick != order.price_tick
+            || order.qty > prev_leaves_qty
+        {
+            let mut cancel_order = order.clone();
+            self.ack_cancel(&mut cancel_order, timestamp)?;
+            self.ack_new(order, timestamp)?;
         } else {
-            // Checks if the sell order price is less than or equal to the current best bid.
-            if exch_order.price_tick <= self.depth.best_bid_tick() {
-                self.sell_orders
-                    .get_mut(&prev_price_tick)
-                    .unwrap()
-                    .remove(&exch_order.order_id);
+            let mut order_borrowed = self.orders.borrow_mut();
+            let exch_order = order_borrowed.get_mut(&order.order_id);
+            let exch_order = exch_order.unwrap();
 
-                if exch_order.time_in_force == TimeInForce::GTX {
-                    exch_order.status = Status::Expired;
-                } else {
-                    // Takes the market.
-                    return self.fill(
-                        &mut exch_order,
-                        timestamp,
-                        false,
-                        self.depth.best_bid_tick(),
-                    );
-                }
-
-                exch_order.exch_timestamp = timestamp;
-                let local_recv_timestamp =
-                    timestamp + self.order_latency.response(timestamp, &exch_order);
-                self.orders_to
-                    .append(exch_order.clone(), local_recv_timestamp);
-                Ok(())
-            } else {
-                // The exchange accepts this order.
-                if prev_price_tick != exch_order.price_tick {
-                    self.sell_orders
-                        .get_mut(&prev_price_tick)
-                        .unwrap()
-                        .remove(&exch_order.order_id);
-                    self.sell_orders
-                        .entry(exch_order.price_tick)
-                        .or_default()
-                        .insert(exch_order.order_id);
-                }
-                if init_q_pos || prev_price_tick != exch_order.price_tick {
-                    // Initialize the order's queue position.
-                    self.queue_model.new_order(&mut exch_order, &self.depth);
-                }
-                exch_order.status = Status::New;
-
-                exch_order.exch_timestamp = timestamp;
-                let local_recv_timestamp =
-                    timestamp + self.order_latency.response(timestamp, &exch_order);
-                self.orders_to
-                    .append(exch_order.clone(), local_recv_timestamp);
-
-                let mut order_borrowed = self.orders.borrow_mut();
-                order_borrowed.insert(exch_order.order_id, exch_order);
-
-                Ok(())
-            }
+            exch_order.qty = order.qty;
+            exch_order.leaves_qty = order.qty;
+            exch_order.exch_timestamp = timestamp;
+            order.leaves_qty = order.qty;
+            order.exch_timestamp = timestamp;
         }
+        Ok(())
     }
 }
 
@@ -741,25 +607,39 @@ where
         timestamp: i64,
         _wait_resp_order_id: Option<OrderId>,
     ) -> Result<bool, BacktestError> {
-        // Processes the order part.
-        while !self.orders_from.is_empty() {
-            let recv_timestamp = self.orders_from.earliest_timestamp().unwrap();
-            if timestamp == recv_timestamp {
-                let (order, _) = self.orders_from.pop_front().unwrap();
-                self.process_recv_order_(order, recv_timestamp)?;
-            } else {
-                assert!(recv_timestamp > timestamp);
-                break;
+        while let Some(mut order) = self.order_e2l.receive(timestamp) {
+            // Processes a new order.
+            if order.req == Status::New {
+                order.req = Status::None;
+                self.ack_new(&mut order, timestamp)?;
             }
+            // Processes a cancel order.
+            else if order.req == Status::Canceled {
+                order.req = Status::None;
+                self.ack_cancel(&mut order, timestamp)?;
+            }
+            // Processes a modify order.
+            else if order.req == Status::Replaced {
+                order.req = Status::None;
+                self.ack_modify::<false>(&mut order, timestamp)?;
+            } else {
+                return Err(BacktestError::InvalidOrderRequest);
+            }
+            // Makes the response.
+            self.order_e2l.respond(order);
         }
         Ok(false)
     }
 
     fn earliest_recv_order_timestamp(&self) -> i64 {
-        self.orders_from.earliest_timestamp().unwrap_or(i64::MAX)
+        self.order_e2l
+            .earliest_recv_order_timestamp()
+            .unwrap_or(i64::MAX)
     }
 
     fn earliest_send_order_timestamp(&self) -> i64 {
-        self.orders_to.earliest_timestamp().unwrap_or(i64::MAX)
+        self.order_e2l
+            .earliest_send_order_timestamp()
+            .unwrap_or(i64::MAX)
     }
 }
