@@ -2,12 +2,16 @@ use std::{ffi::c_void, mem::size_of, ptr::slice_from_raw_parts_mut};
 
 pub use backtest::*;
 pub use depth::*;
+pub use fuse::*;
 #[cfg(feature = "live")]
 use hftbacktest::live::{Instrument, LiveBotBuilder};
 use hftbacktest::{
     backtest::{
+        Asset,
+        Backtest,
+        DataSource,
         assettype::{InverseAsset, LinearAsset},
-        data::{read_npz_file, Data, DataPtr, FeedLatencyAdjustment, Reader},
+        data::{Data, DataPtr, FeedLatencyAdjustment, Reader, read_npz_file},
         models::{
             CommonFees,
             ConstantLatency,
@@ -25,7 +29,7 @@ use hftbacktest::{
             TradingQtyFeeModel,
             TradingValueFeeModel,
         },
-        order::OrderBus,
+        order::order_bus,
         proc::{
             L3Local,
             L3NoPartialFillExchange,
@@ -36,21 +40,24 @@ use hftbacktest::{
             Processor,
         },
         state::State,
-        Asset,
-        Backtest,
-        DataSource,
     },
     prelude::{ApplySnapshot, Event, HashMapMarketDepth, ROIVectorMarketDepth},
 };
 use hftbacktest_derive::build_asset;
 pub use order::*;
-use pyo3::{exceptions::PyValueError, prelude::*};
+use pyo3::{
+    PyTypeInfo,
+    exceptions::{PyDeprecationWarning, PyValueError},
+    ffi::c_str,
+    prelude::*,
+};
 
 #[cfg(feature = "live")]
 use crate::live::{HashMapMarketDepthLiveBot, ROIVectorMarketDepthLiveBot};
 
 mod backtest;
 mod depth;
+mod fuse;
 #[cfg(feature = "live")]
 mod live;
 mod order;
@@ -221,6 +228,8 @@ impl BacktestAsset {
         slf
     }
 
+    /// DEPRECATED: Use `constant_order_latency` instead.
+    ///
     /// Uses `ConstantLatency <https://docs.rs/hftbacktest/latest/hftbacktest/backtest/models/struct.ConstantLatency.html>`_
     /// for the order latency model.
     /// The units of the arguments should match the timestamp units of your data. Nanoseconds are
@@ -233,6 +242,36 @@ impl BacktestAsset {
         mut slf: PyRefMut<Self>,
         entry_latency: i64,
         resp_latency: i64,
+    ) -> PyRefMut<Self> {
+        Python::attach(|py| {
+            PyErr::warn(
+                py,
+                &PyDeprecationWarning::type_object(py),
+                c_str!("constant_latency() is deprecated; use constant_order_latency()."),
+                1,
+            )
+        })
+        .unwrap();
+
+        slf.latency_model = LatencyModel::ConstantLatency {
+            entry_latency,
+            resp_latency,
+        };
+        slf
+    }
+
+    /// Uses `ConstantLatency <https://docs.rs/hftbacktest/latest/hftbacktest/backtest/models/struct.ConstantLatency.html>`_
+    /// for the order latency model.
+    /// The units of the arguments should match the timestamp units of your data. Nanoseconds are
+    /// typically used in HftBacktest.
+    ///
+    /// Args:
+    ///     entry_latency: order entry latency.
+    ///     resp_latency: order response latency.
+    pub fn constant_order_latency(
+        mut slf: PyRefMut<Self>,
+        entry_latency: i64,
+        resp_latency: i64,
     ) -> PyRefMut<Self> {
         slf.latency_model = LatencyModel::ConstantLatency {
             entry_latency,
