@@ -1,14 +1,14 @@
 #![allow(clippy::not_unsafe_ptr_arg_deref)]
 
 use std::{
-    collections::{hash_map::Values, HashMap},
+    collections::{HashMap, hash_map::Values},
     os::raw::c_void,
     ptr::null,
 };
 
 use hftbacktest::prelude::Order;
 
-#[no_mangle]
+#[unsafe(no_mangle)]
 pub extern "C" fn orders_get(ptr: *const HashMap<u64, Order>, order_id: u64) -> *const Order {
     let orders = unsafe { &*ptr };
     match orders.get(&order_id) {
@@ -17,19 +17,19 @@ pub extern "C" fn orders_get(ptr: *const HashMap<u64, Order>, order_id: u64) ->
     }
 }
 
-#[no_mangle]
+#[unsafe(no_mangle)]
 pub extern "C" fn orders_contains(ptr: *const HashMap<u64, Order>, order_id: u64) -> bool {
     let orders = unsafe { &*ptr };
     orders.contains_key(&order_id)
 }
 
-#[no_mangle]
+#[unsafe(no_mangle)]
 pub extern "C" fn orders_len(ptr: *const HashMap<u64, Order>) -> usize {
     let orders = unsafe { &*ptr };
     orders.len()
 }
 
-#[no_mangle]
+#[unsafe(no_mangle)]
 pub extern "C" fn orders_values(ptr: *const HashMap<u64, Order>) -> *mut c_void {
     let orders = unsafe { &*ptr };
     let values = orders.values();
@@ -37,7 +37,7 @@ pub extern "C" fn orders_values(ptr: *const HashMap<u64, Order>) -> *mut c_void
     Box::into_raw(boxed) as *mut _
 }
 
-#[no_mangle]
+#[unsafe(no_mangle)]
 pub extern "C" fn orders_values_next(ptr: *mut Values<u64, Order>) -> *const Order {
     let values = unsafe { &mut *ptr };
     match values.next() {
